<?php

/**
 * @package ecommerce
 */
 
/** 
 *  Configuration mysite/_config.php
 *  ================================
 *  You will firstly need to enable the payment module with something like:
 *  Payment::set_supported_methods(array(
 *    'PaymateExpressPayment ' => 'Credit Card (Paymate)'));
 *
 * Then Set up Paymate account details
 * Set your Paymate merchant ID, this is the id you used to register and login with Paymate
 * PaymateExpressPayment::SetMerchantId("test");
 * // Use _ONE_ of the following Paymate URLs
 * PaymateExpressPayment::SetPaymateUrl("https://www.paymate.com.au/PayMate/ExpressPayment");		// Set Paymate to use LIVE site
 * PaymateExpressPayment::SetPaymateUrl("https://www.paymate.com.au/PayMate/TestExpressPayment");	// Set Paymate to use TEST site
 *
 */

class PaymateExpressPayment extends Payment {
    
    /* Properties */
    
    protected static $merchant_id;
    static function SetMerchantId($id) {
        self::$merchant_id = $id;
    }
    static function GetMerchantId() {
        return self::$merchant_id;
    }

    protected static $paymate_url;
    static function SetPaymateUrl($id) {
        self::$paymate_url = $id;
    }

	static function GetPaymateUrl() {
      return self::$paymate_url;
	}

	protected static $credit_cards = array(
		'Visa' => 'ecommerce/images/payments/methods/visa.jpg',
		'MasterCard' => 'ecommerce/images/payments/methods/mastercard.jpg'
	);


    /* Methods */
    
	function getPaymentFormFields() {
		$logo = '<img src="https://www.paymate.com/PayMate/prep/images/paymate_exp_logo.gif" alt="Credit card payments powered by Paymate" title="Credit card payments powered by Paymate" />';
		$paymentsList = "";
		foreach(self::$credit_cards as $name => $image) $paymentsList .= '<img src="' . $image . '" alt="' . $name . '"/>';
		$fields = new FieldSet(
			new LiteralField('Logo', $logo),
			new LiteralField('CardList', $paymentsList),
			new TextField('Paymate_State', 'State/Region'),
			new TextField('Paymate_Postcode', 'Postcode', '', 7)
		);
		return $fields;

	}
//			new LiteralField("CCblurb","<div id=\"Payment\"><img src=\"https://www.paymate.com/PayMate/prep/images/paymate_exp_logo.gif\" alt=\"Credit card payments powered by Paymate\" title=\"Credit card payments powered by Paymate\" /><br /><img src=\"https://www.worldpay.com/cgenerator/logos/VISA.gif\" alt=\"VISA\" />&nbsp;<img src=\"https://www.worldpay.com/cgenerator/logos/VISD.gif\" alt=\"VISA Delta\" />&nbsp;<img src=\"https://www.worldpay.com/cgenerator/logos/MSCD.gif\" alt=\"MasterCard\" /></div>")

    function getPaymentFormRequirements() {
        return null;
    }
    
    /*
     * Returns the Cheque content from the CheckoutPage
     */
    function ChequeContent() {
        return DataObject::get_one('CheckoutPage')->ChequeMessage;
    }
    
	function processPayment($data, $form) {
		$page = new Page();
		$page->Title = "Make payment";
		$page->Form = $this->PaymateForm($data);
		$page->Logo = '<img src="https://www.paymate.com/PayMate/prep/images/paymate_exp_logo.gif" alt="Credit card payments powered by Paymate" title="Credit card payments powered by Paymate" />';

        //$page->URLSegment = "payment";

		$controller = new Page_Controller($page);
        
		$form = $controller->renderWith("PaymentProcessingPage");
        
		return new Payment_Processing($form);
	}
    
    function PaymateForm($data) {

	$o = $this->Order();
	$items = $o->Items();
	$m = $o->Member();

      //  $m = $this->Member();
      //  $o = DataObject::get_by_id("Order", $this->OrderID);

        //Get the call back URL
        $callbackURL = Director::absoluteBaseURL(). 'PaymateExpressPayment_Handler/paid/' . $this->ID;
        
        $info = array(
		"amt_editable"		=> "N",
		"mid"				=> self::$merchant_id, 
		"ref"				=> $this->ID,
		"currency"			=> $this->Currency,
		"amt"				=> $o->Total(),
		"return"			=> $callbackURL,
		"popup"			=> "flase",
            //
            "pmt_contact_firstname"	=> $m->FirstName,
            "pmt_contact_surname"	=> $m->Surname,
		"pmt_sender_email"	=> $m->Email,
            "pmt_contact_phone"	=> $m->HomePhone,
            "pmt_country"		=> $m->Country,
		//
            "regindi_address1"	=> $m->Address,
            "regindi_address2"	=> $m->AddressLine2,
            "regindi_sub"		=> $m->City,
		"regindi_state"		=> $data['Paymate_State'],	// this Paymate required field is not on the default checkout form
		"regindi_pcode" 		=> $data['Paymate_Postcode']		// this Paymate required field is not on the default checkout form
);

		//if($member->hasMethod('getPostCode')) $info['regindi_pcode'] = $member->getPostCode();
        
        $fields = '';
        foreach($info as $k => $v) {
            $fields .= "<input type=\"hidden\" name=\"" . Convert::raw2att($k) . "\" value=\"" . Convert::raw2att($v) . "\" />\n";
        }
        $merchant_id = self::$merchant_id;
	  $paymate_url = self::$paymate_url;
        return <<<HTML
            <form id="PaymentForm" method="post" action="$paymate_url">
                <h2>Now forwarding you to Paymate...</h2>
                $fields
                <p id="Submitting">We are now forwarding you to Paymate. If you are not redirected in 30 seconds please press the retry button:</p>
                <p class="Actions" id="Submit">
                   <input type="submit" value="Retry" />
                </p>
            </form>
			
		<script>
			$('PaymentForm').submit();
		</script>		

HTML;
    }
    
    function getPaymentFormRequiredFields() {
        return array();
    }
}

/**
 * Handler for responses from the Paymate site
 * @TODO find out how the checkout confirmation page is meant to be displayed - it could be broken in this trunk build
 */
class PaymateExpressPayment_Handler extends Controller {

	function paid() 
		{
		global $project;

		$rp = 'unset';
		$paymentID = $_REQUEST[ref];
		$OrderID = "29";
		$order = DataObject::get_one("Order", "`Order`.ID = $OrderID");
		if($order)
			{
			// $payment = DataObject::get_one("Payment", "`Payment`.`OrderID` = '$OrderID'");		// But what if there's more than one record for some reason?
			$payment = DataObject::get_by_id('PaymateExpressPayment', $paymentID);				// Use an explicit payment ID

			// open a log entry, we'll record the outcomes here
			$logEntry = new OrderStatusLog();

			// common fields for all entries
			$logEntry->OrderID = $OrderID;
			$logEntry->SentToCustomer = 0;

			if(!$payment) // Payment Record does not exist for this order
				{
				$logEntry->Status = 'Payment record attempt: Payment record does not exist for this order, attempting to create one.';

				// open new payment record
				$payment = new Payment();

				// set the common fields
				$payment->OrderID = $OrderID;
				$payment->MemberID = $order->MemberID;
				}

				$payment->TxnRef = $_REQUEST[transactionID];
				$payment->Message = "Paymate Transaction ref: " . $_REQUEST[transactionID];
				$payment->Amount = $_REQUEST[paymentAmount];
				$payment->Currency = $_REQUEST[currency];
				switch ($_REQUEST[responseCode])
					{
					case "PD": // payment was declined
						$payment->Status = 'Failure';
						$logEntry->Status = 'Payment record attempt: Payment was Declined by Paymate';
						$rp = 'incomplete';
						break;
					case "PA": // Payment was Approved
						$order->Status = 'Paid';
						$payment->Status = 'Success';
						$logEntry->Status = 'Payment record attempt: Payment was Approved by Paymate';
						$rp = 'complete';
						$order->write();
						break;
					case "PP": // Payment was authorised and is pending
						$payment->Status = 'Pending';
						$logEntry->Status = 'Payment record attempt: Payment is Pending with Paymate';
						$rp = 'complete';
						break;
					default:
						$payment->Status = 'Failure';
						$logEntry->Status = 'Payment record attempt: Response code not recognised :' . $_REQUEST[responseCode];
						$rp = 'incomplete';
						break;
					}
				$payment->write();
				$logEntry->write();
			}
		else	{
			trigger_error("CheckoutPage::OrderConfirmed - Order not found", E_USER_WARNING);
			return;
			}
		$order->sendReceipt();

/*******************************
THIS SECTION WAS TO SET THE RESPONSE PAGE FOR ECOMMERCE V0.5.2 - NOT SURE HOW THIS IS MEANT TO WORK NOW
********************************
		switch ($rp)
			{
			case "complete":
				//Director::redirect(CheckoutPage::find_link() . "OrderSuccessful/$order->ID");
				break;
			case "incomplete":
				//Director::redirect(CheckoutPage::find_link() . "OrderIncomplete/$order->ID");
				break;
			default:
				//Director::redirect(CheckoutPage::find_link() . "OrderIncomplete/$order->ID");
				break;
			}
*******************************/

		$payment->redirectToOrder(); 	// A POOR SUBSTITUE FOR THE RESPONSE PAGE IN V0.5.2
		}
}

?>